<?php

namespace Teknisa\FIN\Modules\ContaPagar\Services;

use Doctrine\ORM\EntityManager;
use Teknisa\Libs\Util\DateUtil;
use Teknisa\Model\Financeiro\Entities\Movfluxo;
use Teknisa\Model\Financeiro\Entities\Contapagar;
use Teknisa\Model\Financeiro\Entities\Boletpagar;
use Teknisa\Model\Financeiro\Entities\Baixapaga;
use Teknisa\Model\Financeiro\Entities\Tipobaixapag;
use Teknisa\Model\Financeiro\Entities\Movbancario;
use Teknisa\Model\Financeiro\Entities\Cheque;
use Teknisa\Model\Contabilidade\Entities\Lanccont;
use Teknisa\Model\Contabilidade\Entities\Contopectbl;
use Teknisa\FIN\Modules\ContaPagar\Factory\BaixaTituloFactory;
use Teknisa\FIN\Modules\ContaPagar\Services\AjusteTituloService;
use Teknisa\FIN\Modules\FluxoCaixa\Services\FluxoDeCaixaService;
use Teknisa\FIN\Services\ContabilizacaoService;
use Teknisa\Libs\Util\NewCode;
use Teknisa\Libs\Util\Environment;

/**
 * Class BaixaTituloService
 * @package Teknisa\FIN\Modules\ContaPagar\Services
 */
class BaixaTituloService {

    const ID_SIT_CHEQUE_DISP = '0';
    const ID_SIT_CHEQUE_EMIT = '1';
    const BAIXADO_VIA_CNAB   = '90';

    /**
     * @param EntityManager $entityManager
     * @param BaixaTituloFactory $baixaTituloFactory
     * @param AjusteTituloService $ajusteTituloService
     * @param FluxoDeCaixaService $fluxoDeCaixaService
     * @param ContabilizacaoService $contabilizacaoService
     * @param NewCode $newCode
     */
    public function __construct(EntityManager $entityManager, BaixaTituloFactory $baixaTituloFactory, AjusteTituloService $ajusteTituloService, FluxoDeCaixaService $fluxoDeCaixaService, ContabilizacaoService $contabilizacaoService, NewCode $newCode, Environment $environment)
    {
        $this->entityManager         = $entityManager;
        $this->baixaTituloFactory    = $baixaTituloFactory;
        $this->ajusteTituloService   = $ajusteTituloService;
        $this->fluxoDeCaixaService   = $fluxoDeCaixaService;
        $this->contabilizacaoService = $contabilizacaoService;
        $this->newCode               = $newCode;
        $this->environment           = $environment;
    }

    /**
     * Baixa Titulo a Pagar
     * @param $nrorg
     * @param $cdfilial
     * @param $baixaboleto
     * @param $boletosmarcados
     * @throws \Exception
     */
    public function baixaTitulo($nrorg, $cdfilial, $baixaboleto, $boletosmarcados, $contabiliza, $response, $nrlctbbpag = null, $transbaixa = false)
    {
        try {
            // Variáveis de Inicialização
            $nrmovbanc = null;
            $cdoperador = $this->environment->getCdOperador();

            // Formata Data Baixa Paga
            $databaixa = self::formataData($baixaboleto['DTBAIXAPAG']);
            $datalanc  = self::formataData($baixaboleto['DTLANCBAIXAP'], false);
            $idstemicheq = $baixaboleto['IDBAIXAUTILP'] == 'A'?'S':'N';

            // Observação Baixa Paga
            $dsbaixapag  = array_key_exists('DSBAIXAPAG',$baixaboleto)?$baixaboleto['DSBAIXAPAG']:null;

            // Numero,  Nominal e Descrição Cheque
            $nrcheque  = array_key_exists('NRCHEQUE',$baixaboleto)?$baixaboleto['NRCHEQUE']:null;
            $dsnomcheq = array_key_exists('DSNOMCHEQ',$baixaboleto)?$baixaboleto['DSNOMCHEQ']:null;
            $dscheque  = array_key_exists('DSCHEQUE',$baixaboleto)?$baixaboleto['DSCHEQUE']:null;

            // Calcula Valor Total Baixa Paga
            $vrbaixapag = self::valorTotalBaixa($boletosmarcados);

            
            // Verifica Tipo Baixa [Real(E) / Planejada(P) / CNAB(C)]
            if ($baixaboleto['IDSTATUSBPAG'] == 'E'){
                $idstatbpag = 'B'; // Boleto     - Baixado
                $tipofluxo  = 'R'; // Mov. Fluxo - Real
            }else{
                $idstatbpag = 'P'; // Boleto     - Planejado
                $tipofluxo  = 'E'; // Mov. Fluxo - Empenho
            }
            
            // Verifica Status Baixa Paga [Real(E)]
            if ($baixaboleto['IDSTATUSBPAG'] == 'E') {
                
                // Gera Historico de Integração
                if(count($boletosmarcados) > 1)
                     $dshistpadr = self::geraHistoricoIntegracao($boletosmarcados[0]['NRORG'], $boletosmarcados[0]['CDTPBAIXAPAG']);
                else $dshistpadr = self::geraHistoricoIntegracao($boletosmarcados[0]['NRORG'], $boletosmarcados[0]['CDTPBAIXAPAG'], $boletosmarcados[0]['CDFILIAL'], $boletosmarcados[0]['NRLANBOLPAG']);

                // Novo Código Movimentação Bancária
                $nrmovbanc = self::geraNovoCodigo($nrorg, 'MOVBANCARIO'.$baixaboleto['CDREDUCONT']);
                
                // Insere Movimentação Bancária
                /** @var Movbancario $movbancario */
                $movbancario = $this->baixaTituloFactory->insereMovbancario(
                    $nrorg, $nrmovbanc, $baixaboleto['CDREDUCONT'], $databaixa, $vrbaixapag, $dshistpadr
                );

                // Persist Movimentação Bancária
                $this->entityManager->persist($movbancario);
                $this->entityManager->flush();
            }

            // Atualiza Cheque
            if(($baixaboleto['IDBAIXAUTILP'] == 'M') || ($baixaboleto['IDBAIXAUTILP'] == 'A')) {
                /** @var Cheque $cheque */
                $cheque = $this->baixaTituloFactory->buscaCheque(
                    $baixaboleto['CDREDUCONT'], $nrcheque
                );

                // Verifica Existência de Cheque
                if($cheque) {

                    /** @var Cheque $cheque */
                    $cheque = $this->baixaTituloFactory->atualizaCheque(
                        $cheque, self::ID_SIT_CHEQUE_EMIT, $dsnomcheq, $dscheque, $databaixa, $databaixa, $vrbaixapag
                    );
                    $this->entityManager->persist($cheque);
                }
            }

            // Gera Novo Código Baixa
            if(!$transbaixa){
                $nrbaixapag = self::geraNovoCodigo($nrorg, 'BAIXAPAGA'.$cdfilial);
                if(!empty($nrlctbbpag)){
                    $nrlctbbpag = $this->newCode->getCodeByTableName('LANCCONTABIL'.$baixaboleto['CDEMPRESA'].$databaixa->format('Ym'),$nrorg, 1, 12);
                    $nrlctbbpag = $databaixa->format('Ym').substr($nrlctbbpag, 6);
                } 
                // Atualiza Baixa Titulo(s) a Pagar
                /** @var Baixapaga $baixapaga */
                $baixapaga = $this->baixaTituloFactory->insereBaixapaga(
                    $nrorg, $cdfilial, $baixaboleto['CDEMPRESA'], $cdoperador, $baixaboleto['CDTPBAIXAPAG'], $baixaboleto['CDREDUCONT'], $datalanc, $nrmovbanc, $nrbaixapag,
                    $baixaboleto['IDSTATUSBPAG'], $dsbaixapag, $databaixa, $vrbaixapag, $nrcheque, $dsnomcheq, $idstemicheq, $nrlctbbpag
                );
            }else{
                $cdfilial   = $baixaboleto['CDFILIBAIPAG'];
                $nrbaixapag = $baixaboleto['NRBAIXAPAG'];
                $baixapaga = $this->baixaTituloFactory->buscaBaixapaga($nrorg, $cdfilial, $nrbaixapag);
                
                $baixapaga->setIdstatusbpag($baixaboleto['IDSTATUSBPAG']);
                $baixapaga->setNrmovbanc($nrmovbanc);
            }

            $this->entityManager->persist($baixapaga);
            $this->entityManager->flush();
            // Percorre Boletos Pagar
            foreach ($boletosmarcados as $boleto) {
                // Busca Boleto Pagar
                /** @var Boletpagar $boletpagar */
                $boletpagar = $this->baixaTituloFactory->buscaBoletpagar(
                    $boleto['NRORG'], $boleto['CDFILIAL'], $boleto['NRLANBOLPAG']
                );

                if($boletpagar->getNrbaixapag())
                    throw new \Exception('Operação bloqueada. Título baixado por outro usuário.');
                    
                // Atualiza Boleto Pago
                if(isset($boleto['IDSITUCNABP']))
                    if (($boleto['IDSITUCNABP'] == self::BAIXADO_VIA_CNAB) && ($boletpagar->getNrcnabpagret()))
                        $boletpagar->setIdsitucnabp(self::BAIXADO_VIA_CNAB);
                        
                $boletpagar->setCdfilibaipag($cdfilial);
                $boletpagar->setIdstatbpag($idstatbpag);
                $boletpagar->setDtliqbolpag($databaixa);
                $boletpagar->setNrbaixapag($nrbaixapag);
                $boletpagar->setCdtpbaixapag($baixaboleto['CDTPBAIXAPAG']);
                $boletpagar->setCdreducont($baixaboleto['CDREDUCONT']);

                // Persiste Boleto Pago
                $this->entityManager->persist($boletpagar);
                $this->entityManager->flush();
            }
            // Verifica Status Baixa Paga [Real(E)]
            if($baixaboleto['IDSTATUSBPAG'] == 'E') {
                $this->fluxoDeCaixaService->atualizaSaldoContCorr($baixaboleto['CDREDUCONT'], $databaixa, $nrorg, ($vrbaixapag * -1));

                $this->entityManager->flush();

                /** @var Boletpagar $boleto */
                foreach ($boletosmarcados as $boleto) {

                    // Busca Contas a Pagar
                    /** @var Contapagar $contas */
                    $contas = $this->baixaTituloFactory->buscaContaspagarAssoc(
                        $boleto['NRORG'], $boleto['CDFILIAL'], $boleto['NRLANBOLPAG']
                    );

                    //Percorre Contas a Pagar
                    /** @var Contapagar $conta */
                    foreach ($contas as $conta) {
                        
                        // Atualiza Fluxo de Caixa
                        /** @var Movfluxo $movfluxo */
                        $movfluxo = $this->fluxoDeCaixaService->atualizaFluxoCaixa(
                            $nrorg, $conta->getCdempresa(), $conta->getCdfimovflpag(), $conta->getCdclassfina(), $conta->getDtmovfluxo(), $conta->getVrorigpag() * -1, 'E', 'S'
                        );

                        $this->entityManager->persist($movfluxo);
                        $this->entityManager->flush();
                        
                        // Atualiza Fluxo de Caixa
                        /** @var Movfluxo $movfluxo */
                        $movfluxo = $this->fluxoDeCaixaService->atualizaFluxoCaixa(
                            $nrorg, $conta->getCdempresa(), $conta->getCdfimovflpag(), $conta->getCdclassfina(), $databaixa, $conta->getVrorigpag(), 'R', 'S'
                        );

                        $this->entityManager->persist($movfluxo);
                        $this->entityManager->flush();

                        // Atualiza Conta a Pagar
                        /** @var Contapagar $contapagar */
                        $contapagar = $this->baixaTituloFactory->atualizaContapagar(
                            $conta, $idstatbpag, $tipofluxo, $databaixa
                        );

                        $this->entityManager->persist($contapagar);
                        
                        
                        
                        
                        // Atualiza Fluxo dos Ajustes
                        /** @var Ajctpagar $ajustes */
                        $ajustes = $this->baixaTituloFactory->buscaAjctpagar(
                            $conta->getNrorg(), $conta->getCdfilial(), $conta->getNrlanctonf(),
                            $conta->getNrseqvenpag(), $conta->getNrparcelapag()
                        );
                        
                        if ($ajustes){
                            /** @var Ajctpagar $ajuste */
                            foreach ($ajustes as $ajuste) {
                                //Verificação se o Ajuste é de Acréscimo ou Desconto
                                /** @var Tipoajpagar $tipoajuste */
                                $tipoajuste = $this->baixaTituloFactory->buscaTipoajpagar(
                                    $ajuste->getNrorg(), $ajuste->getCdajuctpag()
                                );
    
                                // Desconto - Acrescenta o valor do ajuste na data atual e subtrai na data de Prorrogacao
                                if ($tipoajuste->getIdajuctpag() == 'D') {
                                    $movfluxo = $this->fluxoDeCaixaService->atualizaFluxoCaixa($nrorg, $ajuste->getCdemmfajcntp(), $ajuste->getCdfimfajcntp(),
                                        $ajuste->getCdclassfina(), $ajuste->getDtmovfluxo(), $ajuste->getVrajustepag(), $ajuste->getIdmovfluxo(), 'S');
                                    $this->entityManager->persist($movfluxo);
                                    $this->entityManager->flush();
                                    
                                    $movfluxo = $this->fluxoDeCaixaService->atualizaFluxoCaixa($nrorg, $ajuste->getCdemmfajcntp(), $ajuste->getCdfimfajcntp(),
                                        $ajuste->getCdclassfina(), $databaixa, $ajuste->getVrajustepag() * -1, 'R', 'S');
                                    $this->entityManager->persist($movfluxo);
                                    $this->entityManager->flush();
                                } // Acrescimo - Subtrai o valor do ajuste na data atual e acrescenta na data de Prorrogacao.
                                else {
                                    $movfluxo = $this->fluxoDeCaixaService->atualizaFluxoCaixa($nrorg, $ajuste->getCdemmfajcntp(), $ajuste->getCdfimfajcntp(),
                                        $ajuste->getCdclassfina(), $ajuste->getDtmovfluxo(), $ajuste->getVrajustepag() * -1, $ajuste->getIdmovfluxo(), 'S');
                                    $this->entityManager->persist($movfluxo);
                                    $this->entityManager->flush();
                                    
                                    $movfluxo = $this->fluxoDeCaixaService->atualizaFluxoCaixa($nrorg, $ajuste->getCdemmfajcntp(), $ajuste->getCdfimfajcntp(),
                                        $ajuste->getCdclassfina(), $databaixa, $ajuste->getVrajustepag(), 'R', 'S');
                                    $this->entityManager->persist($movfluxo);
                                    $this->entityManager->flush();
                                }
                                
                                $ajuste->setDtmovfluxo($databaixa);
                                $ajuste->setIdmovfluxo('R');
                                $this->entityManager->persist($ajuste);
                            }
                        }
                        
                        $this->entityManager->flush();
                        
                        
                        
                    }
                }
                
                // Gera Contabilização Baixa Paga
                if($contabiliza === true){
                    $this->contabilizacaoService->contabilizaBaixaPagar(
                        $baixaboleto['CDEMPRESA'], $cdfilial, $nrbaixapag
                    )||null;
                }
                
                // Executa Procedure GRAVA_BAIXA_IMPOSTO
                $this->baixaTituloFactory->execGravaBaixaImposto(
                    $cdfilial, $nrbaixapag, null, null, 'P'
                );
            } else {
                /** @var Boletpagar $boleto */
                foreach ($boletosmarcados as $boleto) {

                    // Busca Contas a Pagar
                    /** @var Contapagar $contaspagar */
                    $contaspagar = $this->baixaTituloFactory->buscaContaspagarAssoc(
                        $boleto['NRORG'], $boleto['CDFILIAL'], $boleto['NRLANBOLPAG']
                    );

                    // Percorre Conta a Pagar
                    /** @var Contapagar $conta */
                    foreach ($contaspagar as $conta) {
                        // Atualiza Fluxo de Caixa
                        /** @var Movfluxo $movfluxo */
                        $movfluxo = $this->fluxoDeCaixaService->atualizaFluxoCaixa(
                            $nrorg, $conta->getCdempresa(), $conta->getCdfimovflpag(), $conta->getCdclassfina(), $conta->getDtmovfluxo(), $conta->getVrorigpag() * -1, $conta->getIdmovfluxo(), 'S'
                        );

                        $this->entityManager->persist($movfluxo);
                        $this->entityManager->flush();

                        // Atualiza Fluxo de Caixa
                        /** @var Movfluxo $movfluxo */
                        $movfluxo = $this->fluxoDeCaixaService->atualizaFluxoCaixa(
                            $nrorg, $conta->getCdempresa(), $conta->getCdfimovflpag(), $conta->getCdclassfina(), $databaixa, $conta->getVrorigpag(), 'E', 'S'
                        );

                        $this->entityManager->persist($movfluxo);
                        $this->entityManager->flush();

                        // Atualiza Conta a Pagar
                        /** @var Contapagar $contapagar */
                        $contapagar = $this->baixaTituloFactory->atualizaContapagar(
                            $conta, $idstatbpag, $tipofluxo, $databaixa
                        );

                        $this->entityManager->persist($contapagar);
                        
                        // Atualiza Fluxo dos Ajustes
                        /** @var Ajctpagar $ajustes */
                        $ajustes = $this->baixaTituloFactory->buscaAjctpagar(
                            $conta->getNrorg(), $conta->getCdfilial(), $conta->getNrlanctonf(),
                            $conta->getNrseqvenpag(), $conta->getNrparcelapag()
                        );
                        
                        if ($ajustes){
                            /** @var Ajctpagar $ajuste */
                            foreach ($ajustes as $ajuste) {
                                //Verificação se o Ajuste é de Acréscimo ou Desconto
                                /** @var Tipoajpagar $tipoajuste */
                                $tipoajuste = $this->baixaTituloFactory->buscaTipoajpagar(
                                    $ajuste->getNrorg(), $ajuste->getCdajuctpag()
                                );
    
                                // Desconto - Acrescenta o valor do ajuste na data atual e subtrai na data de Prorrogacao
                                if ($tipoajuste->getIdajuctpag() == 'D') {
                                    $movfluxo = $this->fluxoDeCaixaService->atualizaFluxoCaixa($nrorg, $ajuste->getCdemmfajcntp(), $ajuste->getCdfimfajcntp(),
                                        $ajuste->getCdclassfina(), $ajuste->getDtmovfluxo(), $ajuste->getVrajustepag(), $ajuste->getIdmovfluxo(), 'S');
                                    $this->entityManager->persist($movfluxo);
                                    $this->entityManager->flush();
                                    
                                    $movfluxo = $this->fluxoDeCaixaService->atualizaFluxoCaixa($nrorg, $ajuste->getCdemmfajcntp(), $ajuste->getCdfimfajcntp(),
                                        $ajuste->getCdclassfina(), $databaixa, $ajuste->getVrajustepag() * -1, 'E', 'S');
                                    $this->entityManager->persist($movfluxo);
                                    $this->entityManager->flush();
                                } // Acrescimo - Subtrai o valor do ajuste na data atual e acrescenta na data de Prorrogacao.
                                else {
                                    $movfluxo = $this->fluxoDeCaixaService->atualizaFluxoCaixa($nrorg, $ajuste->getCdemmfajcntp(), $ajuste->getCdfimfajcntp(),
                                        $ajuste->getCdclassfina(), $ajuste->getDtmovfluxo(), $ajuste->getVrajustepag() * -1, $ajuste->getIdmovfluxo(), 'S');
                                    $this->entityManager->persist($movfluxo);
                                    $this->entityManager->flush();
                                    
                                    $movfluxo = $this->fluxoDeCaixaService->atualizaFluxoCaixa($nrorg, $ajuste->getCdemmfajcntp(), $ajuste->getCdfimfajcntp(),
                                        $ajuste->getCdclassfina(), $databaixa, $ajuste->getVrajustepag(), 'E', 'S');
                                    $this->entityManager->persist($movfluxo);
                                    $this->entityManager->flush();
                                }
                                
                                $ajuste->setDtmovfluxo($databaixa);
                                $ajuste->setIdmovfluxo('E');
                                $this->entityManager->persist($ajuste);
                            }
                        }
                        
                        $this->entityManager->flush();
                    }
                }
            }

            $this->entityManager->flush();
        } catch (\Exception $e) {
            $this->contabilizacaoService->saveContabilizacaoData($response);
            throw $e;
        }
    }


    /**
     * Gera Histórico de Integração
     * @param $nrorg
     * @param $cdreducont
     * @param $dtmovbanc
     * @param $vrmovbanc
     * @return null|string
     * @throws \Exception
     */
    public function geraHistoricoIntegracao($nrorg, $cdtpbaixapag, $cdfilial = null, $nrlanbolpag = null) {
        try{
            $dshistpadr = null;
            $titulo     = [];
            
            // Busca Histórico Padrão
            $historico = $this->baixaTituloFactory->buscaHistoricoPadraoTipoBaixa($nrorg, $cdtpbaixapag, 'BAIXAP');
            
            // Verifica se existe Histórico Padrão
            if(!empty($historico)){
                // Histório Padrão
                $dshistpadr = $historico['DSHISTPADR'];
                
                // Verifica se é somente um título na baixa
                if($cdfilial && $nrlanbolpag){
                    // Busca dados do Título para contrução do Histórico
                    $titulo = $this->baixaTituloFactory->buscaDadosTituloHistorico($nrorg, $cdfilial, $nrlanbolpag);
                    
                    if(!empty($titulo)){
                        
                        // Construção Histórico
                        $dshistpadr = str_replace('[FOR]', $titulo[0]['NRINSJURFORN'], $dshistpadr);
                        $dshistpadr = str_replace('[NFF]', $titulo[0]['NMFANTFORN']  , $dshistpadr);
                        $dshistpadr = str_replace('[RSF]', $titulo[0]['NMRAZSOCFORN'], $dshistpadr);
                        
                        // Verifica se o Título não está associado
                        if(count($titulo) == 1){
                            
                            // Construção Histórico
                            $dshistpadr = str_replace('[FIL]', $titulo[0]['CDFILIRECE']  , $dshistpadr);
                            $dshistpadr = str_replace('[DOC]', $titulo[0]['NRNOTAFISC']  , $dshistpadr);
                            $dshistpadr = str_replace('[SEQ]', $titulo[0]['NRLANCTONF']  , $dshistpadr);
                            $dshistpadr = str_replace('[TCP]', $titulo[0]['DSTPCTPAGAR'] , $dshistpadr);
                            $dshistpadr = str_replace('[DCP]', $titulo[0]['DSBOLETPAG']  , $dshistpadr);
                        }
                    }
                    
                }
            }
            
            if(!$dshistpadr){
                if($cdfilial && $nrlanbolpag)
                    if(count($titulo) > 1)
                         $dshistpadr = 'Baixa Paga - Título Associado';
                    else $dshistpadr = 'Baixa Paga - Título';
                else $dshistpadr = 'Baixa Paga - Múltiplos Título';
            }
            
            return substr($dshistpadr, 0, 40);
        } catch (\Exception $e) {
            throw $e;
        }
    }


    /**
     * Baixa Titulo a Pagar
     * @param $nrorg
     * @param $cdfilial
     * @param $baixaboleto
     * @param $boletosmarcad
     * @throws \Exception
     */
    public function baixaTituloPagar($nrorg, $cdfilial, $baixaboleto, $boletosmarcados, $contabiliza, $response, $nrlctbbpag = null)
    {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            $this->baixaTitulo($nrorg, $cdfilial, $baixaboleto, $boletosmarcados, $contabiliza, $response, $nrlctbbpag, false);
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            $this->contabilizacaoService->saveContabilizacaoData($response);
            throw $e;
        }
    }

    /**
     * Calcula Valor Total Baixa Paga
     * @param $boletosmarcados
     * @return int
     */
    public static function valorTotalBaixa($boletosmarcados)
    {
        $vrmovbanc = 0;

        foreach ($boletosmarcados as $boleto)
            $vrmovbanc += $boleto['VRFINALBPAG'];

        return $vrmovbanc;
    }

    /**
     * Exclusão Baixa Titulo a Pagar
     * @param $nrorg
     * @param $cdfilial
     * @param $boletosmarcados
     * @throws \Exception
     */
    public function excluiBaixa($nrorg, $cdfilial, $boletosmarcados, $dadoscheque = null, $transbaixa = false)
    {
        try {
            // Busca Parâmetros de Títulos a Pagar
            
            /** @var GeralPag $geralpag */
            $geralpag = $this->baixaTituloFactory->buscaGeralPag(
                $nrorg
            );
            
            // Percorre Boletos
            foreach($boletosmarcados as $boletomarcado) {
                
                // Busca Baixa Paga
                /** @var Baixapaga $baixapaga */
                $baixapaga = $this->baixaTituloFactory->buscaBaixapaga($nrorg, $boletomarcado['CDFILIBAIPAG'], $boletomarcado['NRBAIXAPAG']);

                // Verifica Existência Baixa Paga
                if($baixapaga) {

                    // Valida Período Contábil
                    $this->validaPeriodoContabil($baixapaga->getNrorg(), $baixapaga->getCdempresa(), $baixapaga->getDtbaixapag());
                    
                    if(!$transbaixa)
                        // Valida Compensação entre Contas
                        $this->validaCompensacaoConta($baixapaga->getNrorg(), $baixapaga->getCdtpbaixapag());

                    // Valida Adiantamento
                    $this->validaAdiantamento($baixapaga->getNrorg(), $baixapaga->getCdfilial(), $baixapaga->getNrbaixapag());

                    // Valida Movimento Bancario Conciliado
                    $movbanc = $this->validaMovBancarioConciliado($baixapaga->getNrorg(), $baixapaga->getCdreducont(), $baixapaga->getNrmovbanc());

                    // Busca Boletos Baixa Paga
                    /** @var $boletos Boletpagar */
                    $boletos = $this->baixaTituloFactory->buscaBoletsPagarBaixa($baixapaga->getNrorg(), $baixapaga->getCdfilial(), $baixapaga->getNrbaixapag());

                    // Percorre Boleto(s) Baixa Paga
                    /** @var $boleto Boletpagar */
                    foreach ($boletos as $boleto) {

                        // Busca Contas a Pagar
                        /** @var Contapagar $contaspagar */
                        $contaspagar = $this->baixaTituloFactory->buscaContaspagarAssoc(
                            $boleto->getNrorg(), $boleto->getCdfilial(), $boleto->getNrlanbolpag()
                        );

                        // Verifica Existência Conta(s) a Pagar Boleto
                        if ($contaspagar) {
                            // Percorrer Conta(s) a Pagar Boleto
                            /** @var Contapagar $conta */
                            foreach ($contaspagar as $conta) {

                                if($conta->getIdmovfluxo()) {
                                    
                                    // Atualiza Fluxo de Caixa
                                    /** @var Movfluxo $movfluxo */
                                    $movfluxo = $this->fluxoDeCaixaService->atualizaFluxoCaixa(
                                        $nrorg, $conta->getCdempresa(), $conta->getCdfimovflpag(), $conta->getCdclassfina(), $conta->getDtmovfluxo(), $conta->getVrorigpag() * -1, $conta->getIdmovfluxo(), 'S'
                                    );

                                    $this->entityManager->persist($movfluxo);
                                    $this->entityManager->flush();
                                    
                                    // Atualiza Fluxo de Caixa
                                    /** @var Movfluxo $movfluxo */
                                    $movfluxo = $this->fluxoDeCaixaService->atualizaFluxoCaixa(
                                        $nrorg, $conta->getCdempresa(), $conta->getCdfimovflpag(), $conta->getCdclassfina(), $conta->getDtatuavenpag(), $conta->getVrorigpag(), 'E', 'S'
                                    );
                                    $this->entityManager->persist($movfluxo);
                                    $this->entityManager->flush();
                                    
                                    // Atualiza Conta Pagar
                                    $conta->setDtmovfluxo($conta->getDtatuavenpag());
                                    $conta->setIdmovfluxo('E');
                                }

                                // Atualiza Conta Pagar
                                $conta->setIdstatpag('A');
                                $this->entityManager->persist($conta);
                                $this->entityManager->flush();
                                
                                // Atualiza Fluxo dos Ajustes
                                /** @var Ajctpagar $ajustes */
                                $ajustes = $this->baixaTituloFactory->buscaAjctpagar(
                                    $conta->getNrorg(), $conta->getCdfilial(), $conta->getNrlanctonf(),
                                    $conta->getNrseqvenpag(), $conta->getNrparcelapag()
                                );
                                
                                if ($ajustes){
                                    /** @var Ajctpagar $ajuste */
                                    foreach ($ajustes as $ajuste) {
                                        //Verificação se o Ajuste é de Acréscimo ou Desconto
                                        /** @var Tipoajpagar $tipoajuste */
                                        $tipoajuste = $this->baixaTituloFactory->buscaTipoajpagar(
                                            $ajuste->getNrorg(), $ajuste->getCdajuctpag()
                                        );
            
                                        // Desconto - Acrescenta o valor do ajuste na data atual e subtrai na data de Prorrogacao
                                        if ($tipoajuste->getIdajuctpag() == 'D') {
                                            $movfluxo = $this->fluxoDeCaixaService->atualizaFluxoCaixa($nrorg, $ajuste->getCdemmfajcntp(), $ajuste->getCdfimfajcntp(),
                                                $ajuste->getCdclassfina(), $ajuste->getDtmovfluxo(), $ajuste->getVrajustepag(), $ajuste->getIdmovfluxo(), 'S');
                                            $this->entityManager->persist($movfluxo);
                                            $this->entityManager->flush();
                                            
                                            $movfluxo = $this->fluxoDeCaixaService->atualizaFluxoCaixa($nrorg, $ajuste->getCdemmfajcntp(), $ajuste->getCdfimfajcntp(),
                                                $ajuste->getCdclassfina(), $conta->getDtatuavenpag(), $ajuste->getVrajustepag() * -1, 'E', 'S');
                                            $this->entityManager->persist($movfluxo);
                                            $this->entityManager->flush();
                                        } // Acrescimo - Subtrai o valor do ajuste na data atual e acrescenta na data de Prorrogacao.
                                        else {
                                            $movfluxo = $this->fluxoDeCaixaService->atualizaFluxoCaixa($nrorg, $ajuste->getCdemmfajcntp(), $ajuste->getCdfimfajcntp(),
                                                $ajuste->getCdclassfina(), $ajuste->getDtmovfluxo(), $ajuste->getVrajustepag() * -1, $ajuste->getIdmovfluxo(), 'S');
                                            $this->entityManager->persist($movfluxo);
                                            $this->entityManager->flush();
                                            
                                            $movfluxo = $this->fluxoDeCaixaService->atualizaFluxoCaixa($nrorg, $ajuste->getCdemmfajcntp(), $ajuste->getCdfimfajcntp(),
                                                $ajuste->getCdclassfina(), $conta->getDtatuavenpag(), $ajuste->getVrajustepag(), 'E', 'S');
                                            $this->entityManager->persist($movfluxo);
                                            $this->entityManager->flush();
                                        }
                                        
                                        $ajuste->setDtmovfluxo($conta->getDtatuavenpag());
                                        $ajuste->setIdmovfluxo('E');
                                        $this->entityManager->persist($ajuste);
                                    }
                                }
                                
                                $this->entityManager->flush();
                            }
                        }

                        // Atualiza Boleto Pagar
                        if ($boleto->getNrcnabpagrem() == ''){
                            $boleto->setIdsitucnabp('00');
                        }else{
                            $boleto->setIdsitucnabp('10');

                        }

                        $boleto->setIdstatbpag('A');
                        $boleto->setNrbaixapag(NULL);
                        $boleto->setDtliqbolpag(NULL);
                        $boleto->setCdfilibaipag(NULL);
                        
                        if(!$transbaixa){
                            $boleto->setNrcnabpagret(NULL);
                            if($boleto->getNrcnabpagrem())
                                $boleto->setIdsitucnabp('10');
                        }
                        $this->entityManager->persist($boleto);
                        $this->entityManager->flush();
                        
                        
                        if($geralpag->getIdexclajbbxp() == 'S'){
                        
                            // Busca Ajuste(s)
                            /** @var Ajbolpag $ajsbolpag */
                            $ajsbolpag = $this->baixaTituloFactory->buscaAjSbolpag(
                                $boleto->getNrorg(), $boleto->getCdfilial(), $boleto->getNrlanbolpag()
                            );
                            if($ajsbolpag){
                                foreach($ajsbolpag as $ajbolpag){
                                    if(($ajbolpag->getIdorigajbpag() == '2') && ($transbaixa == false)) {
                                        // Busca Tipo de Ajuste
                                        /** @var Tipoajpagar $tipoajpagar */
                                        $tipoajpagar = $this->baixaTituloFactory->buscaTipoajpagar(
                                            $ajbolpag->getNrorg(), $ajbolpag->getCdajuctpag()
                                        );
                                        
                                        $this->ajusteTituloService->deletaAjusteBoleto(
                                            $ajbolpag->getNrorg(), $ajbolpag->getCdfilial(), $ajbolpag->getNrlanbolpag(),
                                            $tipoajpagar->getIdajuctpag(), $ajbolpag->getCdajuctpag(), $ajbolpag->getVrajbolpag()
                                        );
                                    }
                                }
                            }
                        }
                    }
                    
                    // Exclui Contabilizacao Baixa
                    $this->excluiContalizacaoBaixa($baixapaga->getNrorg(), $baixapaga->getCdempresa(), $baixapaga->getNrlctbbpag());
                    $this->entityManager->flush();

                    // Verifica Tipo de Baixa Pagamento [Real]
                    if ($baixapaga->getIdstatusbpag() == 'E') {

                        // Atualiza Saldo Conta Corrente
                        $this->fluxoDeCaixaService->atualizaSaldoContCorr(
                            $baixapaga->getCdreducont(), $baixapaga->getDtbaixapag(), $nrorg, $baixapaga->getVrbaixapag()
                        );

                    }
                    $this->entityManager->flush();

                    // Verifica Existência de Cheque
                    if($baixapaga->getNrcheque()) {
                        /** @var Cheque $cheque */
                        $cheque = $this->baixaTituloFactory->buscaCheque($baixapaga->getCdreducont(), $baixapaga->getNrcheque());

                        // Verifica Existência de Cheque
                        if ($cheque){

                            if(is_array($dadoscheque)) {
                                // Situação e Motivo de Cancelamento
                                $idstcheque = array_key_exists('IDSTCHEQUE', $dadoscheque) ? $dadoscheque['IDSTCHEQUE'] : null;
                                $cdmoticanc = array_key_exists('CDMOTICANC', $dadoscheque) ? $dadoscheque['CDMOTICANC'] : null;
                            }else{
                                $idstcheque = 0;
                                $cdmoticanc = null;
                            }

                            // Atualiza Cheque
                            $cheque = $this->baixaTituloFactory->atualizaCheque(
                                $cheque, $idstcheque, null, null, null, null, null, $cdmoticanc
                            );
                            $this->entityManager->persist($cheque);
                        }
                    }

                    // Exclui Impostos
                    $impbaixapags = $this->baixaTituloFactory->buscaImpbaixapagBaixa($baixapaga->getNrorg(), $baixapaga->getCdfilial(), $baixapaga->getNrbaixapag());
                    if ($impbaixapags)
                        foreach($impbaixapags as $impbaixapag)
                            $this->entityManager->remove($impbaixapag);
                            
                    // Exclui Baixa Pagamento
                    if(!$transbaixa){
                        $this->entityManager->remove($baixapaga);
                    }
                    $this->entityManager->flush();

                    // Verifica Existência Movimentação Baixa Paga
                    if ($movbanc)
                        // Exclui Movimentação Baixa Pagamento
                        $this->entityManager->remove($movbanc);
                        

                    $this->entityManager->flush();
                }
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Exclusão Baixa Titulo a Pagar
     * @param $nrorg
     * @param $cdfilial
     * @param $boletosmarcados
     * @throws \Exception
     */
    public function excluiBaixaPagar($nrorg, $cdfilial, $boletosmarcados, $dadoscheque = null){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            $this->excluiBaixa($nrorg, $cdfilial, $boletosmarcados, $dadoscheque);
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    /**
     * Transforma Baixa Planejado
     * @param $nrorg
     * @param $cdfilial
     * @param $baixaboleto
     * @param $boletosmarcados
     * @param $cheque
     * @param $response
     * @throws \Exception
     */
    public function transformaBaixaPlanejado($nrorg, $cdfilial, $baixaboleto, $boletosmarcados, $cheque, $response){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            //Mudança para não afetar a função devido as alterações para rotina de contabulização
            $contabiliza = true;
            
            $this->excluiBaixa($nrorg, $cdfilial, $boletosmarcados, $cheque, true);
            $this->baixaTitulo($nrorg, $cdfilial, $baixaboleto, $boletosmarcados, $contabiliza, $response, null, true);
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    /**
     * Exclusão Boleto Baixa Titulo
     * @param $nrorg
     * @param $cdfilial
     * @param $boletosmarcados
     * @throws \Exception
     */
    public function excluiTituloBaixa($nrorg, $cdfilial, $boletosmarcados, $response, $transbaixa = false)
    {
        try {
            // Busca Parâmetros de Títulos a Pagar
            /** @var GeralPag $geralpag */
            $geralpag = $this->baixaTituloFactory->buscaGeralPag(
                $nrorg
            );
            
            // Percorre Boletos
            foreach($boletosmarcados as $boletomarcado) {

                // Busca Baixa Paga
                /** @var Baixapaga $baixapaga */
                $baixapaga = $this->baixaTituloFactory->buscaBaixapaga($nrorg, $boletomarcado['CDFILIBAIPAG'], $boletomarcado['NRBAIXAPAG']);

                // Busca Boletos Baixa Paga
                /** @var $boletos Boletpagar */
                $boletos = $this->baixaTituloFactory->buscaBoletsPagarBaixa($baixapaga->getNrorg(), $baixapaga->getCdfilial(), $baixapaga->getNrbaixapag());
                
                // Verifica Existência de Multiplos Boletos
                if(count($boletos) > 1){

                    // Verifica Existência Baixa Paga
                    if($baixapaga) {

                        // Valida Período Contábil
                        $this->validaPeriodoContabil($baixapaga->getNrorg(), $baixapaga->getCdempresa(), $baixapaga->getDtbaixapag());

                        if(!$transbaixa)
                            // Valida Compensação entre Contas
                            $this->validaCompensacaoConta($baixapaga->getNrorg(), $baixapaga->getCdtpbaixapag());
                                             
                        // Valida Adiantamento
                        $baixapag = $this->baixaTituloFactory->validaAdiantamentoMultiploTitulos($boletomarcado['NRORG'], $boletomarcado['CDFILIAL'], $boletomarcado['NRLANBOLPAG']);
                        
                        // Valida Adiantamento Utilizado
                        if ($baixapag)
                        throw new \Exception('Operação bloqueada. Baixa possui adiantamento.');
                        
                        // Valida Movimento Bancario Conciliado
                        $movbanc = $this->validaMovBancarioConciliado($baixapaga->getNrorg(), $baixapaga->getCdreducont(), $baixapaga->getNrmovbanc());


                        // Busca Boletos Baixa Paga
                        /** @var $boletos Boletpagar */
                        $boleto = $this->baixaTituloFactory->buscaBoletPagar($boletomarcado['NRORG'], $boletomarcado['CDFILIAL'], $boletomarcado['NRLANBOLPAG']);
                        
                        // Atualiza Conta a Pagar
                        $this->atualizaContasPagar($boleto, $nrorg);

                        // Atualiza Boleto Pagar
                        if ($boleto->getNrcnabpagrem() == ''){
                            $boleto->setIdsitucnabp('00');
                        }else{
                            $boleto->setIdsitucnabp('10');
                        }

                        $boleto->setIdstatbpag('A');
                        $boleto->setNrbaixapag(NULL);
                        $boleto->setDtliqbolpag(NULL);
                        $boleto->setCdfilibaipag(NULL);
                        
                        if(!$transbaixa){
                            $boleto->setNrcnabpagret(NULL);
                            if($boleto->getNrcnabpagrem())
                                $boleto->setIdsitucnabp('10');
                        }
                        // Exclui Contabilizacao Baixa
                        $this->excluiContalizacaoBaixa($baixapaga->getNrorg(), $baixapaga->getCdempresa(), $baixapaga->getNrlctbbpag());
                        $this->entityManager->flush();

                        // Verifica Tipo de Baixa Pagamento [Real]
                        if ($baixapaga->getIdstatusbpag() == 'E') {
                            // Atualiza Saldo Conta Corrente
                            $this->fluxoDeCaixaService->atualizaSaldoContCorr(
                                $baixapaga->getCdreducont(), $baixapaga->getDtbaixapag(), $nrorg, $boleto->getVrfinalbpag()
                            );
                        }
                        $this->entityManager->flush();

                        // Verifica Existência de Cheque
                        if($baixapaga->getNrcheque()) {
                            $this->atualizaCheque($baixapaga, $boleto);
                        }

                        // Verifica Existência Movimentação Baixa Paga
                        if ($movbanc){
                            // Atualiza Movimentação Baixa Pagamento
                            $movbanc->setVrmovbanc($movbanc->getVrmovbanc() - $boleto->getVrfinalbpag());
                            $this->entityManager->persist($movbanc);
                        }

                        $baixapaga->setVrbaixapag($baixapaga->getVrbaixapag() - $boleto->getVrfinalbpag());
                        $this->entityManager->persist($baixapaga);

                        $this->entityManager->persist($boleto);
                        $this->entityManager->flush();
                        
                        if($geralpag->getIdexclajbbxp() == 'S'){
                            // Busca Ajuste(s)
                            /** @var Ajbolpag $ajsbolpag */
                            $ajsbolpag = $this->baixaTituloFactory->buscaAjSbolpag(
                                $boleto->getNrorg(), $boleto->getCdfilial(), $boleto->getNrlanbolpag()
                            );
                            if($ajsbolpag){
                                foreach($ajsbolpag as $ajbolpag){
                                    if(($ajbolpag->getIdorigajbpag() == '2') && ($transbaixa == false)) {
                                        // Busca Tipo de Ajuste
                                        /** @var Tipoajpagar $tipoajpagar */
                                        $tipoajpagar = $this->baixaTituloFactory->buscaTipoajpagar(
                                            $ajbolpag->getNrorg(), $ajbolpag->getCdajuctpag()
                                        );
                                        
                                        $this->ajusteTituloService->deletaAjusteBoleto(
                                            $ajbolpag->getNrorg(), $ajbolpag->getCdfilial(), $ajbolpag->getNrlanbolpag(),
                                            $tipoajpagar->getIdajuctpag(), $ajbolpag->getCdajuctpag(), $ajbolpag->getVrajbolpag()
                                        );
                                    }
                                }
                            }
                        }
                        
                        // Verifica Tipo de Baixa Pagamento [Real]
                        if ($baixapaga->getIdstatusbpag() == 'E') {
                            // Gera Contabilização Baixa Paga
                            $this->contabilizacaoService->contabilizaBaixaPagar(
                                    $baixapaga->getCdempresa(), $baixapaga->getCdfilial(), $baixapaga->getNrbaixapag()
                            );
                        }
                        
                        // Exclui Impostos
                        $impbaixapags = $this->baixaTituloFactory->buscaImpbaixapagTitulo($baixapaga->getNrorg(), $baixapaga->getCdfilial(), $baixapaga->getNrbaixapag(), $boleto->getCdfilial(), $boleto->getNrlanbolpag());
                        if ($impbaixapags)
                            foreach($impbaixapags as $impbaixapag)
                                $this->entityManager->remove($impbaixapag);
                        
                        $this->entityManager->flush();
                    }
                }else{
                    $this->excluiBaixa($nrorg, $cdfilial, [$boletomarcado], null, $transbaixa);
                }
            }
        } catch (\Exception $e) {
            $this->contabilizacaoService->saveContabilizacaoData($response);
            throw $e;
        }
    }

    /**
     * Exclusão Boleto Baixa Titulo
     * @param $nrorg
     * @param $cdfilial
     * @param $boletosmarcados
     * @throws \Exception
     */
    public function excluiTituloBaixaPagar($nrorg, $cdfilial, $boletosmarcados, $response){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            $this->excluiTituloBaixa($nrorg, $cdfilial, $boletosmarcados, $response);
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    /**
     * Transforma Titulo Planejado
     * @param $nrorg
     * @param $cdfilial
     * @param $baixaboleto
     * @param $boletosmarcados
     * @param $response
     * @throws \Exception
     */
    public function transformaTituloPlanejado($nrorg, $cdfilial, $baixaboleto, $boletosmarcados, $response){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            $contabiliza = true;
            $this->excluiTituloBaixa($nrorg, $cdfilial, $boletosmarcados, $response, false);
            $this->baixaTitulo($nrorg, $cdfilial, $baixaboleto, $boletosmarcados, $contabiliza, $response, null, false);
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    /**
     * Atualiza Contas a Pagar
     * @param $boleto
     * @param $nrorg
     * @throws \Exception
     */
    public function atualizaContasPagar($boleto, $nrorg){
        // Busca Contas a Pagar
        /** @var Contapagar $contaspagar */
        $contaspagar = $this->baixaTituloFactory->buscaContaspagarAssoc(
            $boleto->getNrorg(), $boleto->getCdfilial(), $boleto->getNrlanbolpag()
        );
        // Verifica Existência Conta(s) a Pagar Boleto
        if ($contaspagar) {
            // Percorrer Conta(s) a Pagar Boleto
            /** @var Contapagar $conta */
            foreach ($contaspagar as $conta) {
                if($conta->getIdmovfluxo()) {
                    
                    // Atualiza Fluxo de Caixa
                    /** @var Movfluxo $movfluxo */
                    $movfluxo = $this->fluxoDeCaixaService->atualizaFluxoCaixa(
                        $nrorg, $conta->getCdempresa(), $conta->getCdfimovflpag(), $conta->getCdclassfina(), $conta->getDtmovfluxo(), $conta->getVrorigpag() * -1, $conta->getIdmovfluxo(), 'S'
                    );
                    $this->entityManager->persist($movfluxo);
                    $this->entityManager->flush();
                    
                    // Atualiza Fluxo de Caixa
                    /** @var Movfluxo $movfluxo */
                    $movfluxo = $this->fluxoDeCaixaService->atualizaFluxoCaixa(
                        $nrorg, $conta->getCdempresa(), $conta->getCdfimovflpag(), $conta->getCdclassfina(), $conta->getDtatuavenpag(), $conta->getVrorigpag(), 'E', 'S'
                    );
                    $this->entityManager->persist($movfluxo);
                    $this->entityManager->flush();
                    
                    // Atualiza Conta Pagar
                    $conta->setDtmovfluxo($conta->getDtatuavenpag());
                    $conta->setIdmovfluxo('E');
                }
                // Atualiza Conta Pagar
                $conta->setIdstatpag('A');
                $this->entityManager->persist($conta);
                
                
                // Atualiza Fluxo dos Ajustes
                /** @var Ajctpagar $ajustes */
                $ajustes = $this->baixaTituloFactory->buscaAjctpagar(
                    $conta->getNrorg(), $conta->getCdfilial(), $conta->getNrlanctonf(),
                    $conta->getNrseqvenpag(), $conta->getNrparcelapag()
                );
                
                if ($ajustes){
                    /** @var Ajctpagar $ajuste */
                    foreach ($ajustes as $ajuste) {
                        //Verificação se o Ajuste é de Acréscimo ou Desconto
                        /** @var Tipoajpagar $tipoajuste */
                        $tipoajuste = $this->baixaTituloFactory->buscaTipoajpagar(
                            $ajuste->getNrorg(), $ajuste->getCdajuctpag()
                        );

                        // Desconto - Acrescenta o valor do ajuste na data atual e subtrai na data de Prorrogacao
                        if ($tipoajuste->getIdajuctpag() == 'D') {
                            $movfluxo = $this->fluxoDeCaixaService->atualizaFluxoCaixa($nrorg, $ajuste->getCdemmfajcntp(), $ajuste->getCdfimfajcntp(),
                                $ajuste->getCdclassfina(), $ajuste->getDtmovfluxo(), $ajuste->getVrajustepag(), $ajuste->getIdmovfluxo(), 'S');
                            $this->entityManager->persist($movfluxo);
                            $this->entityManager->flush();
                            
                            $movfluxo = $this->fluxoDeCaixaService->atualizaFluxoCaixa($nrorg, $ajuste->getCdemmfajcntp(), $ajuste->getCdfimfajcntp(),
                                $ajuste->getCdclassfina(), $conta->getDtatuavenpag(), $ajuste->getVrajustepag() * -1, 'E', 'S');
                            $this->entityManager->persist($movfluxo);
                            $this->entityManager->flush();
                        } // Acrescimo - Subtrai o valor do ajuste na data atual e acrescenta na data de Prorrogacao.
                        else {
                            $movfluxo = $this->fluxoDeCaixaService->atualizaFluxoCaixa($nrorg, $ajuste->getCdemmfajcntp(), $ajuste->getCdfimfajcntp(),
                                $ajuste->getCdclassfina(), $ajuste->getDtmovfluxo(), $ajuste->getVrajustepag() * -1, $ajuste->getIdmovfluxo(), 'S');
                            $this->entityManager->persist($movfluxo);
                            $this->entityManager->flush();
                            
                            $movfluxo = $this->fluxoDeCaixaService->atualizaFluxoCaixa($nrorg, $ajuste->getCdemmfajcntp(), $ajuste->getCdfimfajcntp(),
                                $ajuste->getCdclassfina(), $conta->getDtatuavenpag(), $ajuste->getVrajustepag(), 'E', 'S');
                            $this->entityManager->persist($movfluxo);
                            $this->entityManager->flush();
                        }
                        
                        $ajuste->setDtmovfluxo($conta->getDtatuavenpag());
                        $ajuste->setIdmovfluxo('E');
                        $this->entityManager->persist($ajuste);
                    }
                }
                
                $this->entityManager->flush();
            }
        }
    }

    /**
     * Atualiza Cheque
     * @param $baixapaga
     * @param $boleto
     */
    public function atualizaCheque($baixapaga, $boleto){
        /** @var Cheque $cheque */
        $cheque = $this->baixaTituloFactory->buscaCheque($baixapaga->getCdreducont(), $baixapaga->getNrcheque());
        // Verifica Existêncio de Cheque
        if ($cheque){
            // Calcular Valor Atual Cheque
            $vrcheque = $cheque->getVrcheque()-$boleto->getVrfinalbpag();
            $idstcheque   = $vrcheque < 0?$cheque->getIdstcheque():'0';
            $dsnomcheq    = $vrcheque < 0?$cheque->getDsnomcheq():null;
            $dscheque     = $vrcheque < 0?$cheque->getDscheque():null;
            $dtmovbancheq = $vrcheque < 0?$cheque->getDtmovbancheq():null;
            $dtemischeq   = $vrcheque < 0?$cheque->getDtemischeq():null;
            // Atualiza Cheque
            $cheque = $this->baixaTituloFactory->atualizaCheque(
                $cheque, $idstcheque, $dsnomcheq, $dscheque,
                $dtmovbancheq, $dtemischeq, $vrcheque, null
            );
            $this->entityManager->persist($cheque);
        }
    }

    /**
     * Valida Adiantamento
     * @param $nrorg
     * @param $cdempresa
     * @param nrlctbbpag
     * @throws \Exception
     */
    public function excluiContalizacaoBaixa($nrorg, $cdempresa, $nrlctbbpag)
    {
        try {
            // Busca Contábil Baixa Paga
            /** $@var $contabilizacao Lanccont */
            $lancconts = $this->baixaTituloFactory->buscaLanccont($nrorg, $cdempresa, $nrlctbbpag);

            // Exclui Contabilização Baixa Paga
            if ($lancconts)
                foreach($lancconts as $lanccont)
                    $this->entityManager->remove($lanccont);
            
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Valida Adiantamento
     * @param $nrorg
     * @param $cdfilial
     * @param $nrbaixapag
     * @throws \Exception
     */
    public function validaAdiantamento($nrorg, $cdfilial, $nrbaixapag)
    {
        try {
            // Busca Adiantamento Utilizado
            $adiantapagar = $this->baixaTituloFactory->validaAdiantamento(
                $nrorg, $cdfilial, $nrbaixapag
            );

            // Valida Adiantamento Utilizado
            if ($adiantapagar[0]['NRBAIXAPAG'])
                throw new \Exception('Operação bloqueada. Baixa possui adiantamento.');
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Valida Periodo Contábil
     * @param $cdempresa
     * @throws \Exception
     */
    public function validaPeriodoContabil($nrorg, $cdempresa, $dtperiodo)
    {
        try {
            // Formata Data
            $dtperiodo = $this->formataData($dtperiodo)->format('d/m/Y');

            // Busca Controle de Operação Contábil
            /** @var Contopectbl $contopectbl */
            $contopectbl = $this->baixaTituloFactory->validaPeriodoContabil(
                $nrorg, $cdempresa, $dtperiodo
            );

            // Verifica Período Contábil
            if (!$contopectbl)
                throw new \Exception('Operação bloqueada. Data fora do período contábil.');
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Valida Compensacao Entre Contas
     * @param $nrorg
     * @param $cdtpbaixapag
     * @throws \Exception
     */
    public function validaCompensacaoConta($nrorg, $cdtpbaixapag)
    {
        try {
            // Busca Tipo Baixa Paga
            /** @var Tipobaixapag $tipobaixapag */
            $tipobaixapag = $this->baixaTituloFactory->buscaTipobaixapag($nrorg, $cdtpbaixapag);

            // Verifica Compensação Entre Contas
            if ($tipobaixapag->getIdbaixautilp() == 'P')
                throw new \Exception('Operação bloqueada. Baixa compensada entre contas.');
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $nrorg
     * @param $cdreducont
     * @param $nrmovbanc
     * @return null|Movbancario
     * @throws \Exception
     */
    public function validaMovBancarioConciliado($nrorg, $cdreducont, $nrmovbanc)
    {
        try {
            // Busca Movimentação Baixa Paga
            /** @var $movbanc Movbancario */
            $movbanc = $this->baixaTituloFactory->buscaMovbancario($nrorg, $cdreducont, $nrmovbanc);

            // Verifica Existência Movimentação Baixa Paga
            if ($movbanc)
                if ($movbanc->getIdstconext() != 0)
                    throw new \Exception('Operação bloqueada. Movimento bancário da baixa foi conciliado.');

            return $movbanc;
        } catch (\Exception $e) {
            throw $e;
            return null;
        }
    }

    /**
     * Gera Novo Código
     * @param $cdcontador
     * @return bool|null
     * @throws \Exception
     */
    public function geraNovoCodigo($nrorg, $cdcontador){
        try {

            // Gera Novo Código Boleto
            $novocodigo = $this->newCode->getCodeByTableName($cdcontador, $nrorg);

            return $novocodigo;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Formata Data
     * @param $data
     * @param $trunc
     * @return \DateTime|null|\Setforms2\Shared\ORM\DateTime|string
     */
    public function formataData($data, $trunc = true){
        if (!$data)
            return null;

        if (strlen($data) == 19)
            $data = DateUtil::truncateString($data);
        $data = DateUtil::getDataDeString($data);
        
        if($trunc)
            $data = DateUtil::truncateData($data);

        return $data;
    }

}